@@ -42,6 +42,14 @@ extern "C" {
 
 R_LIB_VERSION_HEADER(r_cons);
 
+typedef int (*RConsGetSize)(int *rows);
+typedef int (*RConsGetCursor)(int *rows);
+
+typedef struct r_cons_bind_t {
+	RConsGetSize get_size;
+	RConsGetCursor get_cursor;
+} RConsBind;
+
 typedef struct r_cons_grep_t {
 	char strings[R_CONS_GREP_WORDS][R_CONS_GREP_WORD_SIZE];
 	int nstrings;
@@ -49,7 +57,13 @@ typedef struct r_cons_grep_t {
 	int counter;
 	int less;
 	int json;
+	int range_line;
 	int line;
+	int sort;
+	int sort_row;
+	bool sort_invert;
+	int f_line; //first line
+	int l_line; //last line
 	int tokens[R_CONS_GREP_TOKENS];
 	int tokens_used;
 	int amp;
@@ -110,9 +124,12 @@ typedef struct r_cons_palette_t {
 	char *graph_box;
 	char *graph_box2;
 	char *graph_box3;
+	char *graph_box4;
 	char *graph_true;
 	char *graph_false;
 	char *graph_trufae;
+	char *graph_traced;
+	char *graph_current;
 
 #define R_CONS_PALETTE_LIST_SIZE 8
 	char *list[R_CONS_PALETTE_LIST_SIZE];
@@ -143,6 +160,43 @@ typedef struct r_cons_canvas_t {
 	int linemode; // 0 = diagonal , 1 = square
 } RConsCanvas;
 
+#define RUNECODE_MIN 0xc8 // 200
+#define RUNECODE_LINE_VERT 0xc8
+#define RUNECODE_LINE_CROSS 0xc9
+#define RUNECODE_CORNER_BR 0xca
+#define RUNECODE_CORNER_BL 0xcb
+#define RUNECODE_ARROW_RIGHT 0xcc
+#define RUNECODE_ARROW_LEFT 0xcd
+#define RUNECODE_LINE_HORIZ 0xce
+#define RUNECODE_CORNER_TL 0xcf
+#define RUNECODE_CORNER_TR 0xd0
+#define RUNECODE_LINE_UP 0xd1
+#define RUNECODE_MAX 0xd2
+
+#define RUNECODESTR_MIN 0xc8 // 200
+#define RUNECODESTR_LINE_VERT "\xc8"
+#define RUNECODESTR_LINE_CROSS "\xc9"
+#define RUNECODESTR_CORNER_BR "\xca"
+#define RUNECODESTR_CORNER_BL "\xcb"
+#define RUNECODESTR_ARROW_RIGHT "\xcc"
+#define RUNECODESTR_ARROW_LEFT "\xcd"
+#define RUNECODESTR_LINE_HORIZ "\xce"
+#define RUNECODESTR_CORNER_TL "\xcf"
+#define RUNECODESTR_CORNER_TR "\xd0"
+#define RUNECODESTR_LINE_UP "\xd1"
+#define RUNECODESTR_MAX 0xd1
+
+#define RUNE_LINE_VERT "│"
+#define RUNE_LINE_CROSS "┼" /* ├ */
+#define RUNE_LINE_HORIZ "─"
+#define RUNE_LINE_UP "↑"
+#define RUNE_CORNER_BR "┘"
+#define RUNE_CORNER_BL "└"
+#define RUNE_CORNER_TL "┌"
+#define RUNE_CORNER_TR "┐"
+#define RUNE_ARROW_RIGHT ">"
+#define RUNE_ARROW_LEFT "<"
+
 typedef char *(*RConsEditorCallback)(void *core, const char *file, const char *str);
 typedef int (*RConsClickCallback)(void *core, int x, int y);
 
@@ -154,6 +208,7 @@ r_cons_click_clear();
 
 typedef struct r_cons_t {
 	RConsGrep grep;
+	RStack *cons_stack;
 	char *buffer;
 	//int line;
 	int buffer_len;
@@ -208,6 +263,10 @@ typedef struct r_cons_t {
 #if __WINDOWS__ && !__CYGWIN__
 	bool ansicon;
 #endif
+	bool flush;
+	bool use_utf8; // use utf8 features
+	int linesleep;
+	int pagesize;
 } RCons;
 
 // XXX THIS MUST BE A SINGLETON AND WRAPPED INTO RCons */
@@ -271,6 +330,7 @@ typedef struct r_cons_t {
 #define Color_BBLUE     "\x1b[1;34m"
 #define Color_BGRAY     "\x1b[1;38m"
 
+
 #define Colors_PLAIN { \
 	Color_BLACK, Color_RED, Color_WHITE, \
 	Color_GREEN, Color_MAGENTA, Color_YELLOW, \
@@ -313,17 +373,30 @@ typedef struct r_cons_canvas_line_style_t {
 } RCanvasLineStyle;
 
 // UTF-8 symbols indexes
-
+// XXX. merge with RUNE/RUNECODE/RUNECODESTR
+#if 0
 #define LINE_VERT 0
 #define LINE_CROSS 1
-#define RUP_CORNER 2
-#define RDWN_CORNER 3
-#define ARROW_RIGHT 4
-#define ARROW_LEFT 5
-#define LINE_HORIZ 6
-#define LUP_CORNER 7
-#define LDWN_CORNER 8
-#define LINE_UP 9
+#define LINE_HORIZ 2
+#define LINE_UP 3
+#define CORNER_BR 4
+#define CORNER_BL 5
+#define CORNER_TL 6
+#define CORNER_TR 7
+#define ARROW_RIGHT 8
+#define ARROW_LEFT 9
+#else
+#define LINE_VERT 0
+#define LINE_CROSS 1
+#define LINE_HORIZ 2
+#define LINE_UP 3
+#define CORNER_TL 6
+#define CORNER_BR 4
+#define CORNER_BL 5
+#define CORNER_TR 6
+#define ARROW_RIGHT 8
+#define ARROW_LEFT 9
+#endif
 
 
 #ifdef R_API
@@ -335,7 +408,7 @@ R_API void r_cons_canvas_print_region(RConsCanvas *c);
 R_API char *r_cons_canvas_to_string(RConsCanvas *c);
 R_API void r_cons_canvas_attr(RConsCanvas *c,const char * attr);
 R_API void r_cons_canvas_write(RConsCanvas *c, const char *_s);
-R_API int r_cons_canvas_gotoxy(RConsCanvas *c, int x, int y);
+R_API bool r_cons_canvas_gotoxy(RConsCanvas *c, int x, int y);
 R_API void r_cons_canvas_goto_write(RConsCanvas *c,int x,int y, const char * s);
 R_API void r_cons_canvas_box(RConsCanvas *c, int x, int y, int w, int h, const char *color);
 R_API void r_cons_canvas_line (RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style);
@@ -390,7 +463,8 @@ R_API void r_cons_set_last_interactive(void);
 /* output */
 R_API void r_cons_printf(const char *format, ...);
 R_API void r_cons_strcat(const char *str);
-#define r_cons_puts(x) r_cons_strcat(x)
+#define r_cons_print(x) r_cons_strcat (x)
+R_API void r_cons_println(const char* str);
 R_API void r_cons_strcat_justify (const char *str, int j, char c);
 R_API void r_cons_memcat(const char *str, int len);
 R_API void r_cons_newline(void);
@@ -398,7 +472,7 @@ R_API void r_cons_filter(void);
 R_API void r_cons_flush(void);
 R_API int r_cons_less_str(const char *str, const char *exitkeys);
 R_API void r_cons_less(void);
-R_API void r_cons_2048(void);
+R_API void r_cons_2048(bool color);
 R_API void r_cons_memset(char ch, int len);
 R_API void r_cons_visual_flush(void);
 R_API void r_cons_visual_write (char *buffer);
@@ -422,19 +496,20 @@ R_API const char *r_cons_pal_get_i (int n);
 R_API const char *r_cons_pal_get_color(int n);
 R_API int r_cons_rgb_parse (const char *p, ut8 *r, ut8 *g, ut8 *b, int *is_bg);
 R_API char *r_cons_rgb_tostring(ut8 r, ut8 g, ut8 b);
-R_API void r_cons_pal_list (int rad);
+R_API void r_cons_pal_list (int rad, const char *arg);
 R_API void r_cons_pal_show (void);
 R_API int r_cons_get_size(int *rows);
+R_API bool r_cons_isatty();
 R_API int r_cons_get_cursor(int *rows);
 R_API int r_cons_arrow_to_hjkl(int ch);
 R_API int r_cons_html_print(const char *ptr);
 
 // TODO: use gets() .. MUST BE DEPRECATED
 R_API int r_cons_fgets(char *buf, int len, int argc, const char **argv);
-R_API char *r_cons_hud(RList *list, const char *prompt);
-R_API char *r_cons_hud_path(const char *path, int dir);
-R_API char *r_cons_hud_string(const char *s);
-R_API char *r_cons_hud_file(const char *f);
+R_API char *r_cons_hud(RList *list, const char *prompt, const bool usecolor);
+R_API char *r_cons_hud_path(const char *path, int dir, const bool usecolor);
+R_API char *r_cons_hud_string(const char *s, const bool usecolor);
+R_API char *r_cons_hud_file(const char *f, const bool usecolor);
 
 R_API const char *r_cons_get_buffer(void);
 R_API void r_cons_grep(const char *str);
@@ -456,7 +531,9 @@ R_API void r_cons_column(int c);
 R_API int r_cons_get_column (void);
 R_API char *r_cons_message(const char *msg);
 R_API void r_cons_set_title(const char *str);
-R_API int r_cons_enable_mouse(const int enable);
+R_API bool r_cons_enable_mouse(const bool enable);
+R_API void r_cons_bind(RConsBind *bind);
+R_API const char* r_cons_get_rune(const ut8 ch);
 #endif
 
 /* r_line */
@@ -553,11 +630,9 @@ typedef struct r_ascii_node_t {
 
 #define R_AGRAPH_MODE_NORMAL 0
 #define R_AGRAPH_MODE_OFFSET 1
-#define R_AGRAPH_MODE_ESIL 2
-#define R_AGRAPH_MODE_ESIL_OFFSET 3
-#define R_AGRAPH_MODE_MINI 4
-#define R_AGRAPH_MODE_SUMMARY 5
-#define R_AGRAPH_MODE_MAX 6
+#define R_AGRAPH_MODE_MINI 2
+#define R_AGRAPH_MODE_SUMMARY 3
+#define R_AGRAPH_MODE_MAX 4
 
 typedef void (*RANodeCallback)(RANode *n, void *user);
 typedef void (*RAEdgeCallback)(RANode *from, RANode *to, void *user);
